﻿using System;
using System.Collections.Generic;
using System.Xml;
using UnityEngine;

namespace MouseController
{
    [KSPAddon(KSPAddon.Startup.Flight, false)]
    public class MouseControllerPlugin : MonoBehaviour
    {
        #region Member

        // paths
        public static string Root = KSPUtil.ApplicationRootPath.Replace("\\", "/");
        public static string PluginPath = Root + "GameData/MouseController";
        public static string SettingsPath = PluginPath + "/Data/MouseController_Settings.xml";

        // the interface manager
        private MCInterfaceManager mIManager = null;

        #endregion

        #region public

        // called on load of .dll
        public void Awake()
        {
            mIManager = new MCInterfaceManager();
            mIManager.LoadSettings(SettingsPath);

            WriteErrors2Log();

            RenderingManager.AddToPostDrawQueue(0, Update);
        }

        // called on scene start
        public void Start()
        {
            mIManager.InitializeInterface();

            WriteErrors2Log();
        }

        // called every frame
        public void Update()
        {
            mIManager.UpdateInterface();

            //DrawInterface();
            GUI.DrawTexture(mIManager.SelectedInterface.InterfacePos, mIManager.SelectedInterface.TextureMC_Interface);

            //foreach (MCInterface entry in mSelectedInterface.Subinterfaces.Values)
            //    entry.Update();

            // draw settings window
            if (mIManager.SelectedInterface.ShowSettingsWindow)
            {
                GUI.skin = HighLogic.Skin;
                mIManager.SettingsWindowPos = GUILayout.Window(44834, mIManager.SettingsWindowPos, DrawWindow, "MouseController - Debug Window", GUILayout.MinWidth(200));
            }

            WriteErrors2Log();
        }

        public static Texture2D LoadTexture(string path, MCSize size)
        {
            WWW temp = new WWW("file://" + path.Replace("\\", "/"));
            while (!temp.isDone) { System.Threading.Thread.Sleep(100); }
            Texture2D texture = new Texture2D(size.Width, size.Height);
            temp.LoadImageIntoTexture(texture);

            return texture;
        }

        #endregion

        private void WriteErrors2Log()
        {
            if (mIManager.ErrorList.Count > 0)
            {
                foreach (string errorMsg in mIManager.ErrorList)
                    print(errorMsg);

                mIManager.ErrorList.Clear();
            }
            else
                print("Awake: LoadSettings successfull!");
        }

        private void DrawWindow(int winID)
        {
            switch (winID)
            {
                case 44834: // Debug Window
                    GUILayout.BeginVertical();
                    //GUILayout.Label("Root = " + Root);
                    //GUILayout.Label("PluginFolder = " + PluginFolder);
                    //GUILayout.Label("ButtonPicPath = " + string.Format("file://{0}/Data/MouseController_Button.png", PluginFolder));
                    //GUILayout.Label("InterfacePicPath = " + string.Format("file://{0}/Data/MouseController_Interface.jpg", PluginFolder));
                    //GUILayout.Label("Screen size = (" + mScreenWidth + ", " + mScreenHeight + ")");
                    //GUILayout.Label("MousePosition = (" + mMousePosition.x + ", " + mMousePosition.y + ")");
                    //GUILayout.Label("WindowPosition = (" + mWindowPosition.x + ", " + mWindowPosition.y + ", " + mWindowPosition.width + ", " + mWindowPosition.height + ")");
                    //GUILayout.Label("WindowPosition.Contains = (" + mWindowPosition.Contains(mMousePosition).ToString() + ")");
                    //GUILayout.Label("L Mouse Button = (" + Input.GetMouseButton(0).ToString() + ")");
                    //GUILayout.Label("Rel. Pic Pos = (" + ((int)mMousePosition.x - (int)mWindowPosition.x).ToString() + ", " + ((int)mMousePosition.y - (int)mWindowPosition.y).ToString() + ")");
                    //GUILayout.Label("Texture Pos = (" + ((int)mMousePosition.x - (int)mWindowPosition.x).ToString() + ", " + (mInterfaceHeight - ((int)mMousePosition.y - (int)mWindowPosition.y)).ToString() + ")");
                    //GUILayout.Label("Color = (" + (mCurrentInterfaceStates.Color.r).ToString() + ", " + (mCurrentInterfaceStates.Color.g).ToString() + ", " + (mCurrentInterfaceStates.Color.b).ToString() + ", " + (mCurrentInterfaceStates.Color.a).ToString() + ")");
                    //GUILayout.Label("InterfaceStates:");
                    //GUILayout.Label("ColorArrow = (" + (mColorArrow.r).ToString() + ", " + (mColorArrow.g).ToString() + ", " + (mColorArrow.b).ToString() + ", " + (mColorArrow.a).ToString() + ")");
                    //GUILayout.Label("Arrow (isSameColor)= " + IsSameColor(mCurrentInterfaceStates.Color, mColorArrow).ToString());
                    //GUILayout.Label("HasInput = " + mCurrentInterfaceStates.HasInput.ToString());
                    //GUILayout.Label("Arrow = " + mCurrentInterfaceStates.ArrowDown.ToString());
                    //GUILayout.Label("Func0 = " + mCurrentInterfaceStates.Func0Down.ToString());
                    //GUILayout.Label("Func1 = " + mCurrentInterfaceStates.Func1Down.ToString());
                    //GUILayout.Label("Func2 = " + mCurrentInterfaceStates.Func2Down.ToString());
                    //GUILayout.Label("Func3 = " + mCurrentInterfaceStates.Func3Down.ToString());
                    //GUILayout.Label("Yaw = " + mCurrentInterfaceStates.Yaw.ToString());
                    //GUILayout.Label("Pitch = " + mCurrentInterfaceStates.Pitch.ToString());
                    //GUILayout.Label("Roll = " + mCurrentInterfaceStates.Roll.ToString());
                    //GUILayout.Label("RCS = " + mCurrentInterfaceStates.RCSDown.ToString());
                    //GUILayout.Label("SAS = " + mCurrentInterfaceStates.SASDown.ToString());
                    //GUILayout.Label("NextStage = " + mCurrentInterfaceStates.NextStageDown.ToString());
                    //GUILayout.Label("ThrottleFast = " + mCurrentInterfaceStates.ThrottleMax.ToString());
                    //GUILayout.Label("Throttle = " + mCurrentInterfaceStates.Throttle.ToString());
                    //GUILayout.Label("ThrottleOff = " + mCurrentInterfaceStates.ThrottleOff.ToString());
                    //GUILayout.Label("Actiongroup0 = " + mCurrentInterfaceStates.ActionGroup0Down.ToString());
                    //GUILayout.Label("Actiongroup1 = " + mCurrentInterfaceStates.ActionGroup1Down.ToString());
                    //GUILayout.Label("Actiongroup2 = " + mCurrentInterfaceStates.ActionGroup2Down.ToString());
                    //GUILayout.Label("Actiongroup3 = " + mCurrentInterfaceStates.ActionGroup3Down.ToString());
                    //GUILayout.Label("Actiongroup4 = " + mCurrentInterfaceStates.ActionGroup4Down.ToString());
                    //GUILayout.Label("Actiongroup5 = " + mCurrentInterfaceStates.ActionGroup5Down.ToString());
                    //GUILayout.Label("Actiongroup6 = " + mCurrentInterfaceStates.ActionGroup6Down.ToString());
                    //GUILayout.Label("Actiongroup7 = " + mCurrentInterfaceStates.ActionGroup7Down.ToString());
                    //GUILayout.Label("Actiongroup8 = " + mCurrentInterfaceStates.ActionGroup8Down.ToString());
                    //GUILayout.Label("Actiongroup9 = " + mCurrentInterfaceStates.ActionGroup9Down.ToString());
                    GUILayout.EndVertical();

                    GUI.DragWindow();

                    break;
            }
        }
    }
}
